'use strict'
/* globals describe it beforeEach cli */

let expect = require('chai').expect
let nock = require('nock')
let certs = require('../../../commands/certs/auto.js')

let letsEncrypt = {
  ssl_cert: {
    cert_domains: ['heroku-acm.heroku-cli-sni-test.com', 'heroku-san-test.heroku-cli-sni-test.com'],
    issuer: "/C=US/O=Let's Encrypt/CN=Let's Encrypt Authority X3",
    expires_at: '2012-08-01T21:34:23Z',
    starts_at: '2013-08-01T21:34:23Z',
    subject: '/CN=heroku-acm.heroku-cli-sni-test.com',
    acm: true
  }
}

let selfSigned = {
  ssl_cert: {
    cert_domains: ['heroku-certs-test-update.com'],
    issuer: 'CN=heroku-certs-test-update.com'
  }
}

describe('heroku certs:auto', function () {
  beforeEach(function () {
    cli.mockConsole()
    nock.cleanAll()
  })

  it('displays enabled status message', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [letsEncrypt])
    api.get('/apps/example/domains').reply(200, [
      {'kind': 'heroku', 'hostname': 'tokyo-1050.herokuapp.com', 'cname': null, 'acm_status': null},
      {'kind': 'custom', 'hostname': 'heroku-acm.heroku-cli-sni-test.com', 'cname': 'heroku-acm.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-san-test.heroku-cli-sni-test.com', 'cname': 'heroku-san-test.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'}
    ])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

Certificate details:
Common Name(s): heroku-acm.heroku-cli-sni-test.com
                heroku-san-test.heroku-cli-sni-test.com
Expires At:     2012-08-01 21:34 UTC
Issuer:         /C=US/O=Let's Encrypt/CN=Let's Encrypt Authority X3
Starts At:      2013-08-01 21:34 UTC
Subject:        /CN=heroku-acm.heroku-cli-sni-test.com
SSL certificate is not trusted.

Domain                                   Status
───────────────────────────────────────  ──────
heroku-acm.heroku-cli-sni-test.com       OK
heroku-san-test.heroku-cli-sni-test.com  OK
`)
      api.done()
      apiCerts.done()
    })
  })

  it('displays partially enabled status message', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [letsEncrypt])
    api.get('/apps/example/domains').reply(200, [
      {'kind': 'heroku', 'hostname': 'tokyo-1050.herokuapp.com', 'cname': null, 'acm_status': null},
      {'kind': 'custom', 'hostname': 'heroku-acm.heroku-cli-sni-test.com', 'cname': 'heroku-acm.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-san-test.heroku-cli-sni-test.com', 'cname': 'heroku-san-test.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-in-prog.heroku-cli-sni-test.com', 'cname': 'heroku-in-prog.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'in-progress'},
      {'kind': 'custom', 'hostname': 'heroku-verified.heroku-cli-sni-test.com', 'cname': 'heroku-verified.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'verified'},
      {'kind': 'custom', 'hostname': 'heroku-dns-verified.heroku-cli-sni-test.com', 'cname': 'heroku-dns-verified.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'dns-verified'},
      {'kind': 'custom', 'hostname': 'heroku-missing.heroku-cli-sni-test.com', 'cname': 'heroku-missing.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'failing'},
      {'kind': 'custom', 'hostname': 'heroku-unknown.heroku-cli-sni-test.com', 'cname': 'heroku-unknown.heroku-cli-sni-test.com.herokudns.com', 'acm_status': null}
    ])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

Certificate details:
Common Name(s): heroku-acm.heroku-cli-sni-test.com
                heroku-san-test.heroku-cli-sni-test.com
Expires At:     2012-08-01 21:34 UTC
Issuer:         /C=US/O=Let's Encrypt/CN=Let's Encrypt Authority X3
Starts At:      2013-08-01 21:34 UTC
Subject:        /CN=heroku-acm.heroku-cli-sni-test.com
SSL certificate is not trusted.

Domain                                       Status
───────────────────────────────────────────  ────────────
heroku-acm.heroku-cli-sni-test.com           OK
heroku-san-test.heroku-cli-sni-test.com      OK
heroku-in-prog.heroku-cli-sni-test.com       In Progress
heroku-verified.heroku-cli-sni-test.com      In Progress
heroku-dns-verified.heroku-cli-sni-test.com  DNS Verified
heroku-missing.heroku-cli-sni-test.com       Failing
heroku-unknown.heroku-cli-sni-test.com       Waiting

=== Some domains are failing validation, please verify that your DNS matches: heroku domains
`)
      api.done()
      apiCerts.done()
    })
  })

  it('does not have a false positive check with non-ACM lets encrypt certs', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    let sslCert = Object.assign({}, letsEncrypt.ssl_cert, {acm: false})

    let acmFalse = Object.assign({}, letsEncrypt, {ssl_cert: sslCert})

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [acmFalse])
    api.get('/apps/example/domains').reply(200, [
      {'kind': 'heroku', 'hostname': 'tokyo-1050.herokuapp.com', 'cname': null, 'acm_status': null},
      {'kind': 'custom', 'hostname': 'heroku-acm.heroku-cli-sni-test.com', 'cname': 'heroku-acm.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-san-test.heroku-cli-sni-test.com', 'cname': 'heroku-san-test.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-missing.heroku-cli-sni-test.com', 'cname': 'heroku-missing.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'failing'},
      {'kind': 'custom', 'hostname': 'heroku-unknown.heroku-cli-sni-test.com', 'cname': 'heroku-unknown.heroku-cli-sni-test.com.herokudns.com', 'acm_status': null}
    ])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

Domain                                   Status
───────────────────────────────────────  ───────
heroku-acm.heroku-cli-sni-test.com       OK
heroku-san-test.heroku-cli-sni-test.com  OK
heroku-missing.heroku-cli-sni-test.com   Failing
heroku-unknown.heroku-cli-sni-test.com   Waiting

=== Some domains are failing validation, please verify that your DNS matches: heroku domains
`)
      api.done()
      apiCerts.done()
    })
  })

  it('displays partially enabled status with failed message', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [letsEncrypt])
    api.get('/apps/example/domains').reply(200, [
      {'kind': 'heroku', 'hostname': 'tokyo-1050.herokuapp.com', 'cname': null, 'acm_status': null},
      {'kind': 'custom', 'hostname': 'heroku-acm.heroku-cli-sni-test.com', 'cname': 'heroku-acm.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-san-test.heroku-cli-sni-test.com', 'cname': 'heroku-san-test.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-failed.heroku-cli-sni-test.com', 'cname': 'heroku-failed.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'failed'}
    ])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

Certificate details:
Common Name(s): heroku-acm.heroku-cli-sni-test.com
                heroku-san-test.heroku-cli-sni-test.com
Expires At:     2012-08-01 21:34 UTC
Issuer:         /C=US/O=Let's Encrypt/CN=Let's Encrypt Authority X3
Starts At:      2013-08-01 21:34 UTC
Subject:        /CN=heroku-acm.heroku-cli-sni-test.com
SSL certificate is not trusted.

Domain                                   Status
───────────────────────────────────────  ──────
heroku-acm.heroku-cli-sni-test.com       OK
heroku-san-test.heroku-cli-sni-test.com  OK
heroku-failed.heroku-cli-sni-test.com    Failed

=== Some domains failed validation after multiple attempts, retry by running: heroku certs:auto:refresh
`)
      api.done()
      apiCerts.done()
    })
  })

  it('displays partially enabled status with failing message', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })
    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [letsEncrypt])
    api.get('/apps/example/domains').reply(200, [
      {'kind': 'heroku', 'hostname': 'tokyo-1050.herokuapp.com', 'cname': null, 'acm_status': null},
      {'kind': 'custom', 'hostname': 'heroku-acm.heroku-cli-sni-test.com', 'cname': 'heroku-acm.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-san-test.heroku-cli-sni-test.com', 'cname': 'heroku-san-test.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-failed.heroku-cli-sni-test.com', 'cname': 'heroku-failed.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'failing'}
    ])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

Certificate details:
Common Name(s): heroku-acm.heroku-cli-sni-test.com
                heroku-san-test.heroku-cli-sni-test.com
Expires At:     2012-08-01 21:34 UTC
Issuer:         /C=US/O=Let's Encrypt/CN=Let's Encrypt Authority X3
Starts At:      2013-08-01 21:34 UTC
Subject:        /CN=heroku-acm.heroku-cli-sni-test.com
SSL certificate is not trusted.

Domain                                   Status
───────────────────────────────────────  ───────
heroku-acm.heroku-cli-sni-test.com       OK
heroku-san-test.heroku-cli-sni-test.com  OK
heroku-failed.heroku-cli-sni-test.com    Failing

=== Some domains are failing validation, please verify that your DNS matches: heroku domains
`)
      api.done()
      apiCerts.done()
    })
  })

  it('displays disabled status message', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: false})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [letsEncrypt])
    api.get('/apps/example/domains').reply(200, [])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal('=== Automatic Certificate Management is disabled on example\n')
      api.done()
      apiCerts.done()
    })
  })

  it('displays message that there are no certificates', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [])
    api.get('/apps/example/domains').reply(200, [
      {'kind': 'heroku', 'hostname': 'tokyo-1050.herokuapp.com', 'cname': null, 'acm_status': null},
      {'kind': 'custom', 'hostname': 'heroku-acm.heroku-cli-sni-test.com', 'cname': 'heroku-acm.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'ok'},
      {'kind': 'custom', 'hostname': 'heroku-failing.heroku-cli-sni-test.com', 'cname': 'heroku-failed.heroku-cli-sni-test.com.herokudns.com', 'acm_status': 'failing'}
    ])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

Domain                                  Status
──────────────────────────────────────  ───────
heroku-acm.heroku-cli-sni-test.com      OK
heroku-failing.heroku-cli-sni-test.com  Failing

=== Some domains are failing validation, please verify that your DNS matches: heroku domains
`)
      api.done()
      apiCerts.done()
    })
  })

  it('displays message that there are no domains', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [])
    api.get('/apps/example/domains').reply(200, [
      {'kind': 'heroku', 'hostname': 'tokyo-1050.herokuapp.com', 'cname': null, 'acm_status': null}
    ])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

=== Add a custom domain to your app by running: heroku domains:add <yourdomain.com>
`)
      api.done()
      apiCerts.done()
    })
  })

  it('does not displays message that there are no certificates', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: false})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [])
    api.get('/apps/example/domains').reply(200, [])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal('=== Automatic Certificate Management is disabled on example\n')
      api.done()
      apiCerts.done()
    })
  })

  it('displays message that there are not acm certificates', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: true})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [selfSigned])
    api.get('/apps/example/domains').reply(200, [])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal(
`=== Automatic Certificate Management is enabled on example

=== Add a custom domain to your app by running: heroku domains:add <yourdomain.com>
`)
      api.done()
      apiCerts.done()
    })
  })

  it('does not displays message that there are not acm certificates', function () {
    let api = nock('https://api.heroku.com', {
      reqheaders: {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm'}
    })

    let apiCerts = nock('https://api.heroku.com')

    api.get('/apps/example').reply(200, {acm: false})
    apiCerts.get('/apps/example/sni-endpoints').reply(200, [selfSigned])
    api.get('/apps/example/domains').reply(200, [])

    return certs.run({app: 'example'}).then(function () {
      expect(cli.stderr).to.equal('')
      expect(cli.stdout).to.equal('=== Automatic Certificate Management is disabled on example\n')
      api.done()
      apiCerts.done()
    })
  })
})
